
#ifndef LEXER_H
#define LEXER_H

#include  "buffer.h"
#include  "parse_types.h"
#include  "parser.tab.h"
#include <cctype>
#include <iostream>
#include  <string>
#include  "global.h" 

namespace atlas { namespace interpreter {


class Lexical_analyser
{ enum states  { initial, normal, ended };
BufferedInput& input;
  Hash_table& id_table;
  id_type keyword_limit; // first non-keyword identifier
  id_type type_limit; // first non-type identifier
  int nesting; // number of pending opening symbols
  char prevent_termination; // either |'\0'| or character requiring more input
  int comment_start, comment_end; // characters that start/end a comment
  states state; // to trigger special behaviour
  std::string file_name; // stores a file name for I/O redirection
public:
  Lexical_analyser
    (BufferedInput&, Hash_table&, const char**, const char** type_names);
  int get_token(YYSTYPE *valp, YYLTYPE* locp);
  bool reset(); // get clean slate, return |false| if |getline()| fails
  void set_comment_delims (char c, char d)
          { assert(c!='\0' and d!='\0'); comment_start=c; comment_end=d; }
  const char* scanned_file_name() const { return file_name.c_str(); }
  id_type first_identifier() const { return type_limit; }
  bool is_initial () const { return state==initial; }
private:
  void skip_space() const;
  bool becomes_follows();
  std::string scan_quoted_string() const;
};


extern Hash_table* main_hash_table;

extern Lexical_analyser* lex;


} }
#endif

