
#ifndef PARSETREE_H
#define PARSETREE_H

#include <iostream>
#include <memory>
#include <string>

#include "parse_types.h" // this must precede the next include
#include "parser.tab.h" // because \.{parser.tab.h} does not look after itself
namespace atlas
{
  namespace interpreter
  {

void destroy_expr(expr_p e);

expr_p make_int_denotation (int val, const YYLTYPE& loc);
expr_p make_bool_denotation(bool val, const YYLTYPE& loc);
expr_p make_string_denotation(std::string* val_p, const YYLTYPE& loc);

expr_p make_applied_identifier (id_type id, const YYLTYPE& loc);
expr_p make_dollar(const YYLTYPE& loc);
expr_p make_break(unsigned n,const YYLTYPE& loc);
expr_p make_die(const YYLTYPE& loc);
expr_p make_return(expr_p exp,const YYLTYPE& loc);

raw_expr_list make_exprlist_node(expr_p e, raw_expr_list l);
raw_expr_list reverse_expr_list(raw_expr_list l);
expr_p wrap_tuple_display(raw_expr_list l, const YYLTYPE& loc);
expr_p wrap_list_display(raw_expr_list l, const YYLTYPE& loc);

void destroy_exprlist(raw_expr_list l);

expr_p make_application_node(expr_p f, raw_expr_list args,
 const YYLTYPE& loc, const YYLTYPE& left, const YYLTYPE& right);
expr_p make_application_node(expr_p f, expr_p arg, const YYLTYPE& loc);

expr_p make_unary_call(id_type name, expr_p arg,
   const YYLTYPE& loc, const YYLTYPE& op_loc);
expr_p make_binary_call(id_type name, expr_p x, expr_p y,
   const YYLTYPE& loc, const YYLTYPE& op_loc);

expr_p make_negation(expr_p exp, const YYLTYPE& loc);

raw_form_stack start_formula
  (expr_p e, id_type op, int prio, const YYLTYPE& op_loc);
raw_form_stack start_unary_formula (id_type op, int prio, const YYLTYPE& op_loc);
raw_form_stack extend_formula
  (raw_form_stack pre, expr_p e,id_type op, int prio, const YYLTYPE& op_loc);
expr_p end_formula (raw_form_stack pre, expr_p e, const YYLTYPE& loc);
void destroy_formula(raw_form_stack s);

raw_patlist make_pattern_node(raw_patlist prev,raw_id_pat& pattern);

void destroy_pattern(raw_patlist p);
void destroy_id_pat(const raw_id_pat& p);
raw_patlist reverse_patlist(raw_patlist p);

raw_let_list make_let_node(raw_id_pat& pattern, expr_p val);
raw_let_list append_let_node(raw_let_list prev, raw_let_list cur);
expr_p make_let_expr_node(raw_let_list decls, expr_p body, const YYLTYPE& loc);
std::pair<id_pat,expr> zip_decls(raw_let_list decls);

void destroy_letlist(raw_let_list l);

void destroy_type(type_p t);
void destroy_type_list(raw_type_list t);

expr_p make_lambda_node(raw_patlist pat_l, raw_type_list type_l, expr_p body,
 const YYLTYPE& loc);

expr_p make_conditional_node(expr_p c, expr_p t, expr_p e, const YYLTYPE& loc);

expr_p make_int_case_node(expr_p selector, raw_expr_list ins, const YYLTYPE& loc);

expr_p make_while_node(expr_p b, unsigned flags, const YYLTYPE& loc);
expr_p make_for_node
  (raw_id_pat& id, expr_p ip, expr_p b, unsigned flags, const YYLTYPE& loc);
expr_p make_cfor_node
 (id_type id, expr_p count, expr_p bound, expr_p b, unsigned flags,
  const YYLTYPE& loc);

expr_p make_subscription_node
  (expr_p a, expr_p i, bool reversed, const YYLTYPE& loc);
expr_p make_slice_node
  (expr_p a, expr_p lower, expr_p upper, unsigned flags, const YYLTYPE& loc);

expr_p make_cast(type_p type, expr_p exp, const YYLTYPE& loc);

expr_p make_op_cast(id_type name,type_p type, const YYLTYPE& loc);

expr_p make_assignment(id_type lhs, expr_p rhs, const YYLTYPE& loc);
expr_p make_multi_assignment(raw_id_pat& lhs, expr_p rhs, const YYLTYPE& loc);

expr_p make_comp_ass(expr_p lhs, expr_p rhs, const YYLTYPE& loc);
expr_p make_comp_upd_ass(expr_p lhs, id_type op, expr_p rhs,
   const YYLTYPE& loc,  const YYLTYPE& op_loc);

expr_p make_field_ass(id_type tup, id_type sel, expr_p rhs, const YYLTYPE& loc);
expr_p make_field_upd_ass(expr_p tupex, expr_p selex, id_type op, expr_p rhs,
   const YYLTYPE& loc,  const YYLTYPE& op_loc);

expr_p make_recfun(id_type f, expr_p d,
   const YYLTYPE& loc, const YYLTYPE& f_loc);

expr_p make_sequence
  (expr_p first, expr_p last, unsigned which, const YYLTYPE& loc);

id_type lookup_identifier(const char*);
void include_file(int skip_seen);



std::ostream& operator<<(std::ostream& out, const source_location& sl);

std::ostream& operator<< (std::ostream& out, const expr& e);

bool is_empty(const expr& e);

std::ostream& operator<< (std::ostream& out, const id_pat& p);

  }
}

#endif

