#include  "bitmap.h"
#include  <vector>
#include  <iostream>
#include  <iterator>
#include  <algorithm>


typedef atlas::bitmap::BitMap value_set;

typedef std::vector<value_set> table;

class occurrences
{
  table singles;
  table multiples;

public:
  occurrences (unsigned int max_terms, unsigned long max_sum);

  
  void add(unsigned long term);
  
  const value_set& unique_sums(unsigned int num_terms) const
    { return singles[num_terms]; }

}; 

void print(std::ostream& out, const value_set& s)
{
  std::ostream_iterator<unsigned long> lister(out,",");
  std::copy (s.begin(), s.end(), lister);
  std::cout << std::endl;
}

occurrences::occurrences (unsigned int max_terms, unsigned long max_sum)
: singles(max_terms+1,value_set(max_sum+1))
, multiples(max_terms+1,value_set(max_sum+1))
{ singles[0].insert(0); } 

void occurrences::add(unsigned long term)
{ value_set t (singles[0].capacity());
  value_set d=t; // temporaries to avoid repeated allocation
  for (unsigned i=singles.size(); i-->1; ) // downward loop is important:
  { // at |i-1| we will find sets from before consideration of |term|
    
    t = singles[i-1];
    t<<=term;
    // newly created, maybe unique, sums of |i| terms
    
    d=singles[i];
    d &= t;
    multiples[i] |= d;
    // these are ``new multiple sums''
    singles[i]^= t;
// add new single sums (while removing new multiple sums; not essential)
    
    t = multiples[i-1];
    t<<=term;
    multiples[i] |= t; 
    singles[i].andnot(multiples[i]);
      // remove old or new elements of |multiples[i]| from |singles[i]|
  }
}

int main()
{ const unsigned int k=50;
  const unsigned long last_squared=100;
  unsigned long max = 0;
  for (unsigned long i=1; i<=last_squared; ++i)
    max += i*i;

  std::cout << "Computing unique sums of " << k << " out of the first "
            << last_squared <<  " positive squares." << std::endl 
            << "Using " << 2*(k+1) << " BitMap objects each with capacity of "
            << max+1 << " bits." << std::endl;

  occurrences state(k,max);

  for (unsigned long i=1; i<=last_squared; ++i)
  { std::cout << "Incorporating square of " << i 
              << " which is " << i*i << std::endl; // show progress
    state.add(i*i); // include terms
  }
  const value_set result = state.unique_sums(k);
  std::cout << "Found " << result.size() << " unique sums:" << std::endl;
  print(std::cout,result);
  auto sum = std::accumulate (result.begin(), result.end(), 0);
  std::cout << "Sum is " << sum << '.' << std::endl;
}


